Package: GSEABase
Type: Package
Title: Gene set enrichment data structures and methods
Version: 1.74.0
Authors@R: c(
        person("Martin", "Morgan", role = "aut"),
        person("Seth", "Falcon", role = "aut"),
        person("Robert", "Gentleman", role = "aut"),
        person("Paul", "Villafuerte", role = "ctb",
                comment = "'GSEABase' vignette translation from Sweave to Rmarkdown / HTML"),
        person("Bioconductor Package Maintainer", role = "cre",
                email = "maintainer@bioconductor.org"
        ))
Description: This package provides classes and methods to support Gene
        Set Enrichment Analysis (GSEA).
License: Artistic-2.0
Depends: R (>= 2.6.0), BiocGenerics (>= 0.13.8), Biobase (>= 2.17.8),
        annotate (>= 1.45.3), methods, graph (>= 1.37.2)
Suggests: hgu95av2.db, GO.db, org.Hs.eg.db, Rgraphviz, ReportingTools,
        testthat, BiocStyle, knitr, RUnit
Imports: AnnotationDbi, XML
LazyLoad: yes
Collate: utilities.R AAA.R AllClasses.R AllGenerics.R getObjects.R
        methods-CollectionType.R methods-ExpressionSet.R
        methods-GeneColorSet.R methods-GeneIdentifierType.R
        methods-GeneSet.R methods-GeneSetCollection.R
        methods-OBOCollection.R
VignetteBuilder: knitr
biocViews: GeneExpression, GeneSetEnrichment, GraphAndNetwork, GO, KEGG
git_url: https://git.bioconductor.org/packages/GSEABase
git_branch: RELEASE_3_23
git_last_commit: 00ea6ba
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:23:15 UTC; biocbuild
Author: Martin Morgan [aut],
  Seth Falcon [aut],
  Robert Gentleman [aut],
  Paul Villafuerte [ctb] ('GSEABase' vignette translation from Sweave to
    Rmarkdown / HTML),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Built: R 4.6.0; ; 2026-04-29 04:45:42 UTC; unix
