#Generated by codetoolsBioC version 0.0.9
#Timestamp: Mon Jun 22 15:19:10 2009

#Imports: annotate, AnnotationDbi, Biobase, graph, methods, XML

importClassesFrom(AnnotationDbi, AnnDbBimap, GOAllFrame, KEGGFrame)

importClassesFrom(Biobase, ExpressionSet, ScalarCharacter, Versions)

importClassesFrom(graph, graphNEL)

importClassesFrom(methods, ANY, character, data.frame, environment,
                  factor, list, logical, missing, numeric)

importMethodsFrom(AnnotationDbi, as.list, colnames, get,
                  getGOFrameData, getKEGGFrameData, ls, mappedLkeys,
                  mappedRkeys, mget, nrow, Ontology, orgPackageName,
                  revmap, sample, select, Term, toTable)

importMethodsFrom(Biobase, featureNames)

importMethodsFrom(graph, inEdges, nodes, union)

importMethodsFrom(methods, coerce, initialize, Logic, show)

importMethodsFrom(XML, free, saveXML)

importFrom(annotate, getAnnMap, annPkgName, getEG)

importFrom(AnnotationDbi, orgPackageName)

importFrom(Biobase, mkScalar, selectSome, subListExtract, 
           abstract,
           description, "description<-",
           pubMedIds, "pubMedIds<-")

importFrom(BiocGenerics, updateObject,
           annotation, "annotation<-",
           organism, "organism<-")

importFrom(methods, as, callGeneric, callNextMethod, getClass,
           getSlots, is, new, prototype, representation, selectMethod,
           setClass, slot, "slot<-")

importFrom(utils, head)

importFrom(XML, getNodeSet, xmlAttrs, xmlNode, xmlTreeParse)

exportClasses(## GeneIdentifierType
              GeneIdentifierType,
              NullIdentifier, AnnotationIdentifier,
              ENSEMBLIdentifier, EntrezIdentifier, EnzymeIdentifier,
              GenenameIdentifier, RefseqIdentifier, SymbolIdentifier,
              EntrezIdentifier, UniprotIdentifier, GOAllFrameIdentifier,

              ## CollectionType
              CollectionType, NullCollection, ExpressionSetCollection,
              ComputedCollection,
              CollectionIdType, KEGGCollection, OMIMCollection,
              PMIDCollection, ChrCollection, ChrlocCollection,
              MapCollection, PfamCollection, PrositeCollection,
              GOCollection, OBOCollection,
              BroadCollection,

              ## GeneSet
              GeneSet, GeneColorSet,

              ## GeneSetCollection
              GeneSetCollection
              )

exportMethods(show, coerce, details, initialize, intersect, union,
              setdiff, "&", "|", Logic, incidence,

              ## IdentifierType
              mapIdentifiers, annotation, "annotation<-",

              ## CollectionType
              ids,
              bcCategory, bcSubCategory,
              evidenceCode, ontology,
              subsets, goSlim,

              ## GeneSet
              GeneSet,
              
              geneIdType, geneIds, setIdentifier, setName, description,
              longDescription, organism, pubMedIds, urls, contributor,
              setVersion, creationDate, collectionType,

              "geneIdType<-", "geneIds<-", "setIdentifier<-", "setName<-",
              "description<-", "longDescription<-", "organism<-",
              "pubMedIds<-", "urls<-", "contributor<-",
              "setVersion<-", "creationDate<-", "collectionType<-",

              ## GeneColorSet
              GeneColorSet,
              
              phenotype, geneColor, phenotypeColor, coloring,
              
              "phenotype<-", "geneColor<-", "phenotypeColor<-",
              "coloring<-",

              ## GeneSetCollection
              GeneSetCollection, updateObject, names, "[", "[[")

export(## IdentifierType
       NullIdentifier, ENSEMBLIdentifier, EnzymeIdentifier,
       GenenameIdentifier, RefseqIdentifier, SymbolIdentifier,
       UniprotIdentifier, EntrezIdentifier,
       AnnotationIdentifier, AnnoOrEntrezIdentifier,
       GOAllFrameIdentifier,

       ## CollectionType
       NullCollection, ExpressionSetCollection, ComputedCollection,
       KEGGCollection, OMIMCollection, PMIDCollection, ChrCollection,
       ChrlocCollection, MapCollection, PfamCollection,
       PrositeCollection, GOCollection, OBOCollection, BroadCollection,

       ## XML
       asBroadUri, getBroadSets, toBroadXML,
       getGmt, toGmt,

       ## OBO
       getOBOCollection,

       "geneIds",
       "geneIds<-",
       "ontology"
       )
